# -*- coding: utf-8 -*-
import sqlite3
import os.path
import json
import time
import cPickle

MY_DIR = os.path.dirname(os.path.realpath(__file__))

try:
    from config import ADDON_PROFILE

    DB_PATH = ADDON_PROFILE
except:
    DB_PATH = MY_DIR

DB_FILE = os.path.join(DB_PATH, 'db.sqlite')

CACHE_TIMEOUT = 60 * 60 * 6  # на сколько кэшировать данные, 6 часов


def normalize_torrent_info_url(t_url):
    return '/'.join(t_url.split('/')[3:])


class Database:
    CACHE_PACK = cPickle.dumps  # можно менять функции упаковки в кэш, json pickle и т.п.
    CACHE_UNPACK = cPickle.loads

    def __init__(self):
        self.disable_cache = False
        self.conn = sqlite3.connect(DB_FILE)
        self.conn.text_factory = str
        self.migrate()
        if time.time() - self.db_info.get('last_clean', 0) > CACHE_TIMEOUT:
            self.clean_cache()

    def migrate(self):
        try:
            self.db_info = json.loads(self.conn.execute('SELECT * FROM db_info').fetchone()[0])
            db_version = self.db_info['db_version']
        except sqlite3.OperationalError:
            db_version = 0
        if db_version == 0:
            with self.conn as cur:
                cur.execute('CREATE TABLE db_info (info TEXT)')
                cur.execute('CREATE TABLE cache(cid TEXT, name TEXT, cre_date INTEGER, data TEXT, PRIMARY KEY(cid, name))')
                self.db_info = {'db_version': 1}
                cur.execute('INSERT INTO db_info VALUES (?)', (json.dumps(self.db_info),))
        elif self.db_info['db_version'] == 1:
            with self.conn as cur:
                cur.execute('CREATE TABLE torrent_info(t_url TEXT, status INTEGER DEFAULT 0, data TEXT, PRIMARY KEY(t_url))')
                self.db_info['db_version'] = 2
                self.save_db_info()
        elif self.db_info['db_version'] == 2:
            with self.conn as cur:
                cur.execute('CREATE TABLE favorites(t_url TEXT, params TEXT DEFAULT "", category INTEGER DEFAULT 0, PRIMARY KEY(t_url))')
                self.db_info['db_version'] = 3
                self.save_db_info()

    def favorites_add(self, params):
        with self.conn as cur:
            cur.execute('INSERT OR REPLACE INTO favorites(t_url, params) VALUES (?, ?)', (params['url'], json.dumps(params)))

    def favorites_del(self, t_url):
        with self.conn as cur:
            cur.execute('DELETE FROM favorites WHERE t_url=?', (t_url,))

    def favorites_in(self, t_url):
        return self.conn.execute('SELECT params FROM favorites WHERE t_url=?', (t_url,)).fetchone() is not None

    def get_favorites(self):
        for i in self.conn.execute('SELECT params FROM favorites').fetchall():
            yield json.loads(i[0])

    def torrent_info_get(self, t_url, create_download_task=True):
        t_url = normalize_torrent_info_url(t_url)
        torrent_info = self.conn.execute('SELECT t_url, status, data FROM torrent_info WHERE t_url=?', (t_url,)).fetchone()
        if torrent_info is None:
            if create_download_task:
                with self.conn as cur:
                    cur.execute('INSERT INTO torrent_info VALUES (?, ?, ?)', (t_url, 0, ' '))
            return None
        if torrent_info[1] == 0:
            return None
        return json.loads(torrent_info[2])

    def torrent_info_inactive(self):
        return self.conn.execute('SELECT t_url, status, data FROM torrent_info WHERE status=0').fetchall()

    def torrent_info_update(self, t_url, data):
        t_url = normalize_torrent_info_url(t_url)
        with self.conn as cur:
            cur.execute('UPDATE torrent_info SET data=?, status=1 WHERE t_url=?', (json.dumps(data), t_url))

    def torrent_info_del(self, t_url):
        t_url = normalize_torrent_info_url(t_url)
        with self.conn as cur:
            cur.execute('DELETE FROM torrent_info WHERE t_url=?', (t_url,))

    def clean_cache(self, full=False):
        with self.conn as cur:
            if full:
                cur.execute('DELETE FROM cache')
            else:
                clean_time = int(time.time())
                cur.execute('DELETE FROM cache WHERE ?-cache.cre_date>?', (clean_time, CACHE_TIMEOUT))
                self.db_info['last_clean'] = clean_time
                self.save_db_info()

    def save_db_info(self):
        with self.conn as cur:
            cur.execute('UPDATE db_info SET info=?', (json.dumps(self.db_info),))

    def get_cache(self, name, cid):
        data = self.conn.execute('SELECT data FROM cache WHERE name=? AND cid=?', (name, cid)).fetchone()
        if data is None:
            return None
        return self.CACHE_UNPACK(data[0])

    def set_cache(self, name, cid, obj):
        with self.conn as cur:
            cur.execute('INSERT OR REPLACE INTO cache(name, cid, cre_date, data) VALUES (?, ?, ?, ?)', (name, cid, int(time.time()), self.CACHE_PACK(obj)))

    def del_cache(self, name, cid):
        with self.conn as cur:
            cur.execute('DELETE FROM cache WHERE name=? AND cid=?', (name, cid))

    def cached(self, name):
        """Декоратор кэша, по тэгу name и id в кэше - последний аргумент кэшируемой функции"""

        def get_worker_f(forker_f):
            def get_cid(*args):
                if self.disable_cache:
                    cache = None
                else:
                    cache = self.get_cache(name, args[-1])
                if cache is not None:
                    return cache
                data = forker_f(*args)
                self.set_cache(name, args[-1], data)
                return data

            return get_cid

        return get_worker_f


def test():
    db = Database()
    db.clean_cache(full=True)
    print db.get_cache('test', '1')
    db.set_cache('test', '1', {'obj': 777})
    print db.get_cache('test', '1')['obj']
    db.del_cache('test', '1')

    @db.cached('test')
    def test_gen_data(cid):
        print 'worker generate data'
        return {'obj': 888, 'cid': cid}

    print test_gen_data('1')
    print test_gen_data('1')
    print test_gen_data('2')
    db.torrent_info_get('http://open-tor.org/torrent/615574/otstavnik-pozyvnoj-«brodjaga»-2017-webrip-avc-ot-new-team')
    print db.torrent_info_inactive()
    print db.favorites_in('test')


if __name__ == '__main__':
    test()
